#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import sys
import os
import koprfunc
import matplotlib.pyplot as plt
from PyQt5 import QtCore, QtGui, QtWidgets
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.widgets import Slider, Button, RadioButtons
from matplotlib.figure import Figure

class VisObsCanvas(FigureCanvas):

    def __init__(self, parent=None, width=5, height=4, dpi=100, xlist=[], ylist=[], t=[], s=[], mag=0.0, title='', afrad=[], afrho=[] ):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = self.fig.add_subplot(111)
        if mag == -100:                
          self.axes.plot(xlist, ylist, 'ro', t, s, 'b')
          labels = [ 'Radius (km)','Magnitude', 0 ]
        else:
          self.axes.plot(xlist, ylist, 'ro', 0, mag, 'bs', t, s, 'b')
          labels = [ 'Argelander degree','Magnitude', min(xlist)-1 ]
        
        self.axes.grid(color='k', linestyle='-', linewidth=1)
        self.axes.set_title(title)
        self.axes.set_xlabel(labels[0])
        self.axes.set_ylabel(labels[1])
        self.axes.set_xlim(labels[2], max(xlist)+1)
        self.axes.set_ylim(max(ylist)+1, min(ylist)-1)

        if afrho:
            self.ax1 = self.axes.twinx()
            self.ax1.plot(afrad, afrho, 'b^')
            self.ax1.set_ylabel('Afrho (cm)', color='b')
            self.ax1.tick_params('y', colors='b', direction='in')        
                
        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

    def update_figure(self):
        self.axes.cla()
        self.axes.plot(self.t, self.s)
        self.draw()
    
    def save_figure(self, fname):
        self.fig.savefig(fname)

class CompCanavas(FigureCanvas):

    def __init__(self, parent=None, width=10, height=8, dpi=100, plots=[]):
        fig = Figure(figsize=(width, height), dpi=dpi)
        
        self.mags = fig.add_subplot(211)
        xpt, ypt = plots[0], plots[1]
        self.mags.plot(xpt[0], ypt[0], color='red', marker='o', ls='none', label='Your obs.')
        self.mags.plot(xpt[1], ypt[1], color='black', marker='v', ls='none', label='CCD')
        self.mags.plot(xpt[2], ypt[2], color='black', marker='*', ls='none', label='Visual')
        self.mags.plot(xpt[3], ypt[3], color='black', label=plots[2] ) 
        self.mags.set_ylabel('Total coma magnitude')
        self.mags.set_xlim(plots[3], plots[4])
        self.mags.get_xaxis().set_visible(False)
        self.mags.invert_yaxis()

        self.coma = fig.add_subplot(212)
        self.coma.plot(xpt[4], ypt[4], color='red', marker='o', ls='none')
        self.coma.plot(xpt[5], ypt[5], color='black', marker='v', ls='none')
        self.coma.plot(xpt[6], ypt[6], color='black', marker='*', ls='none')        
        self.coma.set_xlabel('Date')
        self.coma.set_ylabel('Coma diameter')
        self.coma.set_xlim(plots[3], plots[4])
        for tick in self.coma.get_xticklabels():
            tick.set_rotation(30)

        fig.tight_layout()
        FigureCanvas.__init__(self, fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

class VisMapDialog(FigureCanvas):

    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi, facecolor='black' )
        self.ax = fig.add_subplot(111, aspect='equal', facecolor='black')

        #load background image
        self.ax.set_xlim(0, 1024*2)
        self.ax.set_ylim(768*2, 0)
        self.ax.set_facecolor('black')

        #no ticks        
        self.ax.get_xaxis().set_visible(False)
        self.ax.get_yaxis().set_visible(False)
        
        # scale
        self.scalept = self.ax.plot([], [], marker='|', color = 'red', lw=1)[0]
        self.scaletext = self.ax.text(1024*2-40-341, 768*2-60, '', color='red',  ha='center')

        # limit mag
        self.limmag = self.ax.text(0, 0, 'Limit mag.: ', color='red', fontweight='bold', va='bottom', ha='left', transform=self.ax.transAxes, fontsize=8)

        # compass
        self.ax.text(0.5, 1, 'N', color='red', fontweight='bold', va='top', ha='center', transform=self.ax.transAxes)
        self.ax.text(1, 0.5, 'W', color='red', fontweight='bold', ha='right', va='center', transform=self.ax.transAxes)
        self.ax.text(0, 0.5, 'E', color='red', fontweight='bold', va='center', transform=self.ax.transAxes)
        self.ax.text(0.5, 0, 'S', color='red', fontweight='bold', ha='center', transform=self.ax.transAxes)

        # comet pointer
        self.com_point = self.ax.plot([], [], marker='+', color = 'red',linestyle='none')[0]

        # plot coma
        self.pt_plot = self.ax.plot([], [], marker='+', color = 'red')[0]
        
        # plot tail
        self.tpt_plot = self.ax.plot([], [], marker='+', color = 'red')[0]
        
        # plot refstars 
        self.starcirc = {}
        self.circle = []
        
        fig.tight_layout(pad = 0)
        
        FigureCanvas.__init__(self, fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Preferred,
                                   QtWidgets.QSizePolicy.Preferred)
        FigureCanvas.updateGeometry(self)

        self.canvas = self.ax.get_figure().canvas
        self.canvas.setFocusPolicy( QtCore.Qt.ClickFocus )
        self.canvas.setFocus()

class ImageDialog(FigureCanvas):

    def __init__(self, parent=None, width=5, height=4, dpi=100, image_data = [] ):
        self.fig = Figure(figsize=(width, height), dpi=dpi, facecolor='gray')
        self.ax = self.fig.add_subplot(111, aspect='equal')

        image_data, datamin, datamax, median = image_data

        # ------------- Comet poiner addon START
        self.com_plot = self.ax.plot([], [], marker='x', color = 'white')[0]
        
        # plot refstars 
        self.starcirc = {}

        # plot tail
        self.tpt_plot = self.ax.plot([], [], marker='+', color = 'red')[0]

        # plot starmap
        self.bluplt = self.ax.plot( [], [], color='blue', marker='s', markersize=8, ls = 'None', fillstyle='none' )[0]
        self.yelplt = self.ax.plot( [], [], color='orange', marker='s', markersize=8, ls = 'None', fillstyle='none' )[0]
        self.redplt = self.ax.plot( [], [], color='red', marker='s', markersize=8, ls = 'None', fillstyle='none' )[0]
        self.graplt = self.ax.plot( [], [], color='ivory', marker='s', markersize=8, ls = 'None', fillstyle='none' )[0]

        # plot astrometry
        self.real_plot = self.ax.plot([], [], marker='x', color = 'green', markersize=8, ls = 'None', fillstyle='none')[0]
        self.mapa_plot = self.ax.plot([], [], marker='x', color = 'blue', markersize=8, ls = 'None', fillstyle='none')[0]

        self.img = self.ax.imshow(image_data, cmap='gray', clim=([datamin, datamax]) )
        self.ax.axes.get_xaxis().set_ticks([])
        self.ax.axes.get_yaxis().set_ticks([])
        
        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        FigureCanvas.updateGeometry(self)
        self.fig.tight_layout()

        self.fig.subplots_adjust(bottom=0.15)
        self.axmin = self.fig.add_axes([0.15, 0.05, 0.7, 0.03], facecolor='lightgoldenrodyellow')
        self.axmax = self.fig.add_axes([0.15, 0.10, 0.7, 0.03], facecolor='lightgoldenrodyellow')

        self.smin = Slider(self.axmin, 'Min', 0, median, valinit=datamin)
        self.smax = Slider(self.axmax, 'Max', median, datamax*5., valinit=datamax)

        self.canvas = self.ax.get_figure().canvas
        self.canvas.setFocusPolicy( QtCore.Qt.ClickFocus )
        self.canvas.setFocus()

    def AxRefresh(self, image_data):
        image_data, datamin, datamax, median = image_data
        
        self.ax.clear()

        # ------------- Comet poiner addon 
        self.com_plot = self.ax.plot([], [], marker='x', color = 'white')[0]
        
        # plot tail
        self.tpt_plot = self.ax.plot([], [], marker='+', color = 'red')[0]
        
        # plot starmap
        self.bluplt = self.ax.plot( [], [], color='blue', marker='s', markersize=8, ls = 'None', fillstyle='none' )[0]
        self.yelplt = self.ax.plot( [], [], color='orange', marker='s', markersize=8, ls = 'None', fillstyle='none' )[0]
        self.redplt = self.ax.plot( [], [], color='red', marker='s', markersize=8, ls = 'None', fillstyle='none' )[0]
        self.graplt = self.ax.plot( [], [], color='ivory', marker='s', markersize=8, ls = 'None', fillstyle='none' )[0]

        # plot astrometry
        self.real_plot = self.ax.plot([], [], marker='x', color = 'green', markersize=8, ls = 'None', fillstyle='none')[0]
        self.mapa_plot = self.ax.plot([], [], marker='x', color = 'blue', markersize=8, ls = 'None', fillstyle='none')[0]

        self.img = self.ax.imshow(image_data, cmap='gray', clim=([datamin, datamax]) )
        self.ax.axes.get_xaxis().set_ticks([])
        self.ax.axes.get_yaxis().set_ticks([])

class LightCurve(FigureCanvas):

    def __init__(self, parent=None, width=5, height=4, dpi=100 ):
        self.fig = Figure(figsize=(width, height), dpi=dpi, facecolor='lightgray')
        self.ax = self.fig.add_subplot(111, facecolor='lightgray')

        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        FigureCanvas.updateGeometry(self)
        self.fig.tight_layout()

        self.canvas = self.ax.get_figure().canvas
        self.canvas.setFocusPolicy( QtCore.Qt.ClickFocus )
        self.canvas.setFocus()

class MPCCanavas(FigureCanvas):

    def __init__(self, parent=None, width=5, height=4, dpi=100):
        self.fig = Figure(figsize=(width, height), dpi=dpi)
        self.ax = self.fig.add_subplot(111)
        
        #self.lc = self.ax.plot( plots[0][0], plots[0][1], color='black', lw = 1, aa = True )[0]
        #self.excl = self.ax.plot( plots[1][0], plots[1][1], color='black', marker='*', markersize = 4, ls = 'None', label='Excluded' )[0]
        #self.sel = self.ax.plot( plots[2][0], plots[2][1], color='red', marker='x', markersize = 4, ls = 'None', label='Selected' )[0]
        #self.ax.legend(loc=1)
        #self.ax.tick_params(labelsize=8)
        #self.ax.grid()
        
        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

        self.canvas = self.ax.get_figure().canvas
        self.canvas.setFocusPolicy( QtCore.Qt.ClickFocus )
        self.canvas.setFocus()
                
